package bluetoothIM.Protocol.Conversations;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.UUID;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothSocket;
import android.os.Handler;
import bluetoothIM.Protocol.Connections.InputOutput;
import bluetoothIM.Protocol.Connections.Listening;
import bluetoothIM.Protocol.Routing.RoutingAlgorithm;
import bluetoothIM.Protocol.Routing.RoutingHandler;
import bluetoothIM.Protocol.Routing.RoutingTable;

public class Room { // Tda reprezentujc virtuln mstnost
	
	// Get/Set metody
	public static HashMap<String, Conversation> getConversations() {
		return Conversations;
	}

	public static void setConversations(
			HashMap<String, Conversation> conversations) {
		Conversations = conversations;
	}

	public static ArrayList<String> getMembers() {
		return Members;
	}

	public static void setMembers(ArrayList<String> members) {
		Members = members;
	}


	public static Handler getHandler() {
		return handler;
	}

	public static void setHandler(Handler handler) {
		Room.handler = handler;
	}

	private static Handler handler;
	private static HashMap<String, Conversation> Conversations;
	private static ArrayList<String> Members;

	static { // Konstruktor
		Conversations = new HashMap<String, Conversation>();
		Members = new ArrayList<String>();

	}

	public static void newMessageReceived(String fromAddress, String fromName, // Zpracovn nov zprvy
			String text, UUID id) {
		Message m = new Message(fromAddress, fromName, text, id);
		Conversations.get(fromAddress).getInMessageConversation().add(m);

		if (fromAddress.equals("Everyone")) {
			Conversations.get(fromAddress).refreshEveryoneConversation();
			handler.obtainMessage(3 + Members.indexOf("Everyone"),
					Conversations.get(fromAddress).getInStringConversation())
					.sendToTarget();
		}

		else {
			Conversations.get(fromAddress).refreshConversation();
			handler.obtainMessage(
					3 + Members.indexOf(RoutingTable.getAddressNamePairs().get(
							fromAddress)),
					Conversations.get(fromAddress).getInStringConversation())
					.sendToTarget();
		}
	}

	public static void sendMessageTo(String name, String text) { // Odesln nov zprvy
		String toAddress;
		if (name.endsWith("Everyone"))
			toAddress = "Everyone";
		else
			toAddress = RoutingTable.getNameAddressPairs().get(name);

		Message m = new Message(toAddress, BluetoothAdapter.getDefaultAdapter()
				.getName(), text, true, UUID.randomUUID());
		RoutingHandler.sendMessageTo(toAddress, text, m.getID());
		Conversations.get(toAddress).getToDeliver().put(m.getID(), m);
		Conversations.get(toAddress).getInMessageConversation().add(m);
		Conversations.get(toAddress).refreshConversation();
		handler.obtainMessage(3 + Members.indexOf(name),
				Conversations.get(toAddress).getInStringConversation())
				.sendToTarget();

	}

	public static void delivered(String address, UUID deliveredID) { // Zpracovn potvrzen
		if (getConversations().containsKey(address)) {
			getConversations().get(address).wasDelivered(deliveredID);
			getConversations().get(address).refreshConversation();
			handler.obtainMessage(
					3 + Members.indexOf(RoutingTable.getAddressNamePairs().get(
							address)),
					Conversations.get(address).getInStringConversation())
					.sendToTarget();
		}
	}

	public static void delivered(String everyone, String sourceAddress, // Zpracovn potvrzen hromadn zprvy
			UUID deliveredID) {
		if (getConversations().containsKey(everyone)) {
			Conversations.get(everyone)
					.wasDelivered(sourceAddress, deliveredID);
			Conversations.get(everyone).refreshEveryoneConversation();
			handler.obtainMessage(3 + Members.indexOf(everyone),
					Conversations.get(everyone).getInStringConversation())
					.sendToTarget();
		}
	}

	public static void onConnect(BluetoothSocket socket) { // Metoda reagujc na vytvoen novho spojen
		
		String s = socket.getRemoteDevice().getName()+ " has connected";
		handler.obtainMessage(-1, s).sendToTarget();
	
		RoutingTable.addAddressListAddressPair(socket.getRemoteDevice()
				.getAddress(), BluetoothAdapter.getDefaultAdapter()
				.getAddress());
		RoutingTable.addAddressNamePair(socket.getRemoteDevice().getAddress(),
				socket.getRemoteDevice().getName());
		RoutingTable.addNameAddressPair(socket.getRemoteDevice().getName(),
				socket.getRemoteDevice().getAddress());
		RoutingTable.addAddressDirectlyConnectedPair(socket.getRemoteDevice()
				.getAddress(), socket);
		RoutingTable.addMemberName(socket.getRemoteDevice().getName());
		newConversation(socket.getRemoteDevice().getAddress());
		InputOutput.getStream(socket);
		RoutingAlgorithm.run();
		for (String x : RoutingTable.getTable().keySet())
		{
			RoutingHandler.SendUpdate(x);
			
		}
		getRoomMembers();
		if(RoutingHandler.deleted.contains(socket.getRemoteDevice().getAddress()))
		RoutingHandler.deleted.remove(socket.getRemoteDevice().getAddress());
	}

	public static void getRoomMembers() { // Obnoven seznamu len mstnosti
		Members.clear();
		for(String s : RoutingTable.getTable().keySet())
        {
            if (s != BluetoothAdapter.getDefaultAdapter().getAddress() && !Members.contains(RoutingTable.getAddressNamePairs().get(s)))
                Members.add(RoutingTable.getAddressNamePairs().get(s));
        }
		if (Members.size() > 1 && !Members.contains("Everyone")) {
			Members.add("Everyone");
			if (!Conversations.containsKey("Everyone"))
				Conversations.put("Everyone", new Conversation());
		}
		
		handler.obtainMessage(2, Members).sendToTarget();

	}

	private static void newConversation(String address) { // Vytvoen nov konverzace
		Conversations.put(address, new Conversation());

	}

}
