﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using InTheHand;
using InTheHand.Net.Bluetooth;
using InTheHand.Net.Ports;
using InTheHand.Net.Sockets;
using BluetoothIM.Model.Protocol.Routing;
using BluetoothIM.Model.Protocol.Connections;
using System.Collections.ObjectModel;


namespace BluetoothIM.Model.Protocol.Conversations
{
    public class Room // Třída reprezentující virtuální místnost
    {
        public static Dictionary<string, Conversation> Conversations { get; set; }
        public static List<string> Members { get; set; }
        public static event OnNewMessage message;
        public delegate void OnNewMessage(string address);
        public static event OnConnect connectionDone;
        public delegate void OnConnect();
        public static event NotConnected connectionFailed;
        public delegate void NotConnected();
        public static event setStatus statusEvent;
        public delegate void setStatus();
        public static string Status { get; set; }
        public static void refreshConversation(string address) // Metody vyvolávající událost po kontrole, zda je někdo přihlášen k jejich odběru
        {
            if (message != null)
            {
                message(address);
            }
        }
        public static void failed()
        {
                getRoomMembers();
            if (connectionFailed != null)
                connectionFailed();
        }
        public static void connected()
        {
                getRoomMembers();
            if (connectionDone != null)
                connectionDone();
        }
        public static void status()
        {
            if (statusEvent != null)
            {
                statusEvent();
            }
        }
        static Room() // Konstruktor
        {
            Conversations = new Dictionary<string, Conversation>();
            Members = new List<string>();
        }
        public static void newMessageReceived(string fromAddress, string fromName, string text, Guid id) // Přijetí nové zprávy
        {
            Message m = new Message(fromAddress, fromName, text, id);
            Conversations[fromAddress].InMessageConversation.Add(m);
            if (fromAddress == "Everyone")
            {
                Conversations[fromAddress].refreshEveryoneConversation();
                refreshConversation(fromAddress);
            }
            else 
            {
                Conversations[fromAddress].refreshConversation();
                refreshConversation(fromAddress);
            }

        }
        public static void sendMessageTo(string name, string text) // Poslání nové zprávy
        {
            string toAddress ;
            if(name == "Everyone")
                toAddress  = "Everyone";
            else
                toAddress = RoutingTable.NameAddressPairs[name];
            Message m = new Message(toAddress, BluetoothRadio.PrimaryRadio.Name, text, true, Guid.NewGuid());
            RoutingHandler.sendMessageTo(toAddress, text, m.ID);
            Conversations[toAddress].ToDeliver.Add(m.ID, m);
            Conversations[toAddress].InMessageConversation.Add(m);
            Conversations[toAddress].refreshConversation();

            refreshConversation(toAddress);
        }
        public static void delivered(string address, Guid deliveredID) // Příjem potvrzení
        {
            if (Conversations.ContainsKey(address))
            {
                Conversations[address].wasDelivered(deliveredID);
                Conversations[address].refreshConversation();
                refreshConversation(address);
            }
        }
        public static void delivered(string everyone, string fromAddress, Guid deliveredID) // Příjem hromadného potvrzení
        {
            if (Conversations.ContainsKey(everyone))
            {
                Conversations[everyone].wasDelivered(fromAddress, deliveredID);
                Conversations[everyone].refreshEveryoneConversation();
                refreshConversation(everyone);
            }
        }
        public static async void onConnect(BluetoothClient client) // Metoda volaná při připojení nového zařízení
        {
            Status = client.RemoteMachineName + "\nhas connected";
            status();
            RoutingTable.addAddressListAddressPair(client.RemoteEndPoint.Address.ToString("C"), BluetoothRadio.PrimaryRadio.LocalAddress.ToString("C"));
            RoutingTable.addAddressNamePair(client.RemoteEndPoint.Address.ToString("C"), client.RemoteMachineName);
            RoutingTable.addNameAddressPair(client.RemoteMachineName, client.RemoteEndPoint.Address.ToString("C"));
            RoutingTable.addAddressDirectlyConnectedPair(client.RemoteEndPoint.Address.ToString("C"), client);
            RoutingTable.addMemberName(client.RemoteMachineName);
            newConversation(client.RemoteEndPoint.Address.ToString("C"));
          
            await Task.Run(() => InputOutput.GetStream(client));
            RoutingAlgorithm.run();
            foreach (string s in Routing.RoutingTable.Table.Keys.ToArray())
            {
                if (s != BluetoothRadio.PrimaryRadio.LocalAddress.ToString("C") && s != null)
                    Routing.RoutingHandler.SendUpdate(s);
            }
            getRoomMembers();
           
            connected();
            if (RoutingHandler.deleted.Contains(client.RemoteEndPoint.Address.ToString("C")))
                RoutingHandler.deleted.Remove(client.RemoteEndPoint.Address.ToString("C"));
        }

      
        public static void getRoomMembers() // Obnovení seznamu členů místnosti
        {
            Members.Clear();
            foreach (string s in RoutingTable.Table.Keys.ToArray())
            {
                if (s != BluetoothRadio.PrimaryRadio.LocalAddress.ToString("C") && !Members.Contains(RoutingTable.AddressNamePairs[s]))
                    Members.Add(RoutingTable.AddressNamePairs[s]);
            }
            if (Members.Count > 1 && !Members.Contains("Everyone"))
            {
                Members.Add("Everyone");
                if (!Conversations.ContainsKey("Everyone"))
                    Conversations.Add("Everyone", new Conversation());
            }
         
        }
        private static void newConversation(string address) // Vytvoření nové konverzace
        {
            if(!Conversations.ContainsKey(address))
            Conversations.Add(address, new Conversation());
        }

       
    }
}
