﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using InTheHand;
using InTheHand.Net.Bluetooth;
using InTheHand.Net.Ports;
using InTheHand.Net.Sockets;

namespace BluetoothIM.Model.Protocol.Routing
{
    class RoutingAlgorithm // Třída pro výpočet cest
    {

        public static void run() // Metoda, ve která běží algoritmus pro výpočet cesty
        {
            RoutingTable.Table.Clear();
            List<string> toFind = new List<string>();
            foreach (string s in RoutingTable.AddressListAddressPairs.Keys)
                toFind.Add(s);
            Dictionary<string, Dictionary< BluetoothClient,int>> costTable = new Dictionary<string, Dictionary<BluetoothClient,int>>();
            Dictionary<string, List<string>> topology = new Dictionary<string, List<string>>();
            toFind.Remove(BluetoothRadio.PrimaryRadio.LocalAddress.ToString("C"));
            getTopology(topology);

            foreach (var a in RoutingTable.AddressDirectlyConnectedPairs) // Všechna přímo připojená zařízení jsou přidána do tabulky
            {
                if (toFind.Contains(a.Key))
                {
                    costTable.Add(a.Key, new Dictionary<BluetoothClient,int>());
                    costTable[a.Key].Add(a.Value,1);
                    toFind.Remove(a.Key);
                }
            }
            if (toFind.Count == 0) // Pokud jsou všichni přímo připojení je výpočet ukončen
            {
                setTable(costTable);
                return;
            }
            
            foreach (var b in RoutingTable.AddressDirectlyConnectedPairs) // Pro každého souseda je spuštěno vyhledávání ve stromu
            {
                int i = 1;
                getTopology(topology);
                removeDirectlyConnected(topology);
                find(toFind, topology, b.Key, b.Key, b.Value, i, costTable);
            }
            setTable(costTable); 

        }

        private static void setTable(Dictionary<string, Dictionary<BluetoothClient,int>> costTable) // Zkopírování dočasné tabulky do výsledné
        {
            foreach (string s in costTable.Keys)
            {
                foreach (var r in costTable[s].Keys)
                 {
                    if(!RoutingTable.Table.ContainsKey(s))
                    RoutingTable.Table.Add(s, r);
                }
            }
        }

        private static void getTopology(Dictionary<string, List<string>> topology) // Vytvoření topologie pro výpočet
        {
            topology.Clear();
            foreach (string s in RoutingTable.AddressListAddressPairs.Keys)
            {

                topology.Add(s, new List<string>());
                foreach (string p in RoutingTable.AddressListAddressPairs[s])
                {
                    topology[s].Add(p);
                }
            }
            removeNode(topology, BluetoothRadio.PrimaryRadio.LocalAddress.ToString("C"));
        }
        private static void find(List<string> toFindList, Dictionary<string, List<string>> topology, // Prohledávání topologie a přidávání nalezených cest do tabulky
        string node, string hop, BluetoothClient client, int i, Dictionary<string, Dictionary<BluetoothClient,int>> costTable)
        {
            if (topology[node].Count != 0)
            {
                i++;
                foreach (var a in topology[node])
                {
                    
                    if (toFindList.Contains(a))
                    {
                         if(!costTable.ContainsKey(a)) // Pokud je nalezena kratší cesta je přidána do dočasné tabulky
                            costTable.Add(a, new Dictionary< BluetoothClient,int>()); 
                            if (!costTable[a].ContainsKey(client))
                                costTable[a].Add(client,i); 
                            if (costTable[a][client] > i)
                                costTable[a][client] = i;
                      
                        
                    }
                    removeNode(topology, node);// Odebrání zpáteční cesty 
                    find(toFindList, topology, a, hop, client, i, costTable); // Pro každého souseda spuštění znovu
                }

            }
            else
                return;

        }
        private static void removeNode(Dictionary<string, List<string>> topology, string node) // Odstranění uzlu z topologie
        {
            topology.Remove(node);
            foreach (var a in topology.Keys)
            {
                if (topology[a].Contains(node))
                {
                    topology[a].Remove(node);
                }
            }

        }
        private static void removeDirectlyConnected(Dictionary<string, List<string>> topology) // Odstranění přímo připojených z topologie
        {
            foreach (var b in RoutingTable.AddressDirectlyConnectedPairs)
            {
              
        
            foreach (var a in topology.Keys)
            {
                if (topology[a].Contains(b.Key))
                {
                    topology[a].Remove(b.Key);
                }
            }
        }

        }


    }
}
