﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using InTheHand;
using InTheHand.Net.Bluetooth;
using InTheHand.Net.Ports;
using InTheHand.Net.Sockets;
using BluetoothIM.Model.Protocol.Routing.Packets;
namespace BluetoothIM.Model.Protocol.Routing
{
    public class RoutingTable // Třída reprezentující směrovací tabulku
    {
        public static Dictionary<string, List<string>> AddressListAddressPairs { get; set; }
        public static Dictionary<string, string> AddressNamePairs { get; set; }
        public static Dictionary<string, string> NameAddressPairs { get; set; }
        public static Dictionary<string, BluetoothClient> AddressDirectlyConnectedPairs { get; set; }
        public static Dictionary<string, BluetoothClient> Table { get; set; }
        public static List<string> MemberNames { get; set; }

        static RoutingTable() // Konstruktor
        {
            AddressListAddressPairs = new Dictionary<string, List<string>>();
            AddressNamePairs = new Dictionary<string, string>();
            AddressNamePairs.Add(BluetoothRadio.PrimaryRadio.LocalAddress.ToString("C"), BluetoothRadio.PrimaryRadio.Name);
            NameAddressPairs = new Dictionary<string, string>();
            NameAddressPairs.Add(BluetoothRadio.PrimaryRadio.Name, BluetoothRadio.PrimaryRadio.LocalAddress.ToString("C"));
            AddressDirectlyConnectedPairs = new Dictionary<string, BluetoothClient>();
            Table = new Dictionary<string, BluetoothClient>();
            MemberNames = new List<string>();
            addMemberName(BluetoothRadio.PrimaryRadio.Name);
        }

        public static void addAddressListAddressPair(string addressOne, string addressTwo) // Metody pro přidávání a odebírání zařízení ze slovníků
        {
            if (!AddressListAddressPairs.ContainsKey(addressOne))
            {
                AddressListAddressPairs.Add(addressOne, new List<string>());
            }
            AddressListAddressPairs[addressOne].Add(addressTwo);
            if (!AddressListAddressPairs.ContainsKey(addressTwo))
            {
                AddressListAddressPairs.Add(addressTwo, new List<string>());
            }
            AddressListAddressPairs[addressTwo].Add(addressOne);
        }
        public static void removeAddressListAddressPair(string addressOne, string addressTwo)
        {
            if (AddressListAddressPairs.ContainsKey(addressOne))
            {
                AddressListAddressPairs[addressOne].Remove(addressTwo);
                if (AddressListAddressPairs[addressOne].Count == 0)
                    AddressListAddressPairs.Remove(addressOne);
            }


            if (AddressListAddressPairs.ContainsKey(addressTwo))
            {
                AddressListAddressPairs[addressTwo].Remove(addressOne);
                if (AddressListAddressPairs[addressTwo].Count == 0)
                    AddressListAddressPairs.Remove(addressTwo);
            }
        }
        public static void addAddressNamePair(string address, string name)
        {
            if (!AddressNamePairs.ContainsKey(address))
                AddressNamePairs.Add(address, name);
        }
        public static void removeAddressNamePair(string address)
        {
            if (AddressNamePairs.ContainsKey(address) && !AddressListAddressPairs.ContainsKey(address))
                AddressNamePairs.Remove(address);
        }
        public static void addNameAddressPair(string name, string address)
        {
            if (!NameAddressPairs.ContainsKey(name))
                NameAddressPairs.Add(name, address);
        }
        public static void removeNameAddressPair(string name)
        {
            if (NameAddressPairs.ContainsKey(name) && !AddressListAddressPairs.ContainsKey(NameAddressPairs[name]))
                NameAddressPairs.Remove(name);
        }
        public static void addAddressDirectlyConnectedPair(string address, BluetoothClient client)
        {
            if (!AddressDirectlyConnectedPairs.ContainsKey(address))
                AddressDirectlyConnectedPairs.Add(address, client);
        }
        public static void removeAddressDirectlyConnectedPair(string address)
        {
            if (AddressDirectlyConnectedPairs.ContainsKey(address))
                AddressDirectlyConnectedPairs.Remove(address);
        }
        public static void addToTable(string address, BluetoothClient client)
        {
            if (!Table.ContainsKey(address))
                Table.Add(address, client);
        }
        public static void removeFromTable(string address)
        {
            if (Table.ContainsKey(address))
                Table.Remove(address);
        }

        public static void addMemberName(string name)
        {
            if (!MemberNames.Contains(name))
            {
                MemberNames.Add(name);
            }
        }
        public static void removeMemberName(string name)
        {
            if (MemberNames.Contains(name))
            {
                MemberNames.Remove(name);
            }
        }

       public static int getAddressPairCount()
        {
            int i = 0;
            foreach (string s in AddressListAddressPairs.Keys)
            {
                foreach (string t in AddressListAddressPairs[s])
                {
                    i++;
                }
            }
            return i;
        }
    }
}
