﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using BluetoothIM;
using System.ComponentModel;
using System.Windows;
using InTheHand;
using InTheHand.Net.Bluetooth;
using InTheHand.Net.Ports;
using InTheHand.Net.Sockets;
using System.Threading;
using BluetoothIM.Model.Protocol;
using System.Collections.ObjectModel;
using BluetoothIM.ViewModel.Commands;
using BluetoothIM.Model.Protocol.Connections;
using BluetoothIM.Model.Protocol.Routing;
using BluetoothIM.Model.Protocol.Conversations;

namespace BluetoothIM.ViewModel
{
    public class IMViewModel : INotifyPropertyChanged // Třída sloužící jako datacontext 
    {
        public Connect _Connect { get; set; }
        public Close _Close { get; set; }
        public Scan _Scan { get; set; }
        public OnWidth _OnWidth { get; set; }
        public OffWidth _OffWidth { get; set; }
        public Mini _Mini { get; set; }
        public Send _Send { get; set; }

  
        private List<string> roomMembers;
        private List<string> inRangeCollection;
        private string inputText;
        private string selectedMember;
        private int windowWidth;
        private string outputText;
        private int toConnect;
        private string enabled;
        private string windowState;
        private string conversation;
        private string scanText;
        private IMViewModelLogic logic;
        private string connection;
      
        public IMViewModel() // Konstruktor
        {
            WindowWidth = 210;
            Enabled = "Hidden";
            roomMembers = new List<string>();
            inRangeCollection = new List<string>();
            Connection = "";
           
            _Connect = new Connect(this);
            _Close = new Close(this);
            _Scan = new Scan(this);
            _OnWidth = new OnWidth(this);
            _OffWidth = new OffWidth(this);
            _Mini = new Mini(this);
            _Send = new Send(this);
            Thread t = new Thread(OnOff);
            t.Start();
           
        }
      
       
        public event PropertyChangedEventHandler PropertyChanged; // Událost při změně vlastnosti
        private void OnPropertyChanged(string propertyName) // Zpracování změny vlastnosti
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        } 
        // Get/Set metody pro vlastnosti
        public IMViewModelLogic Logic
        {
            get { return logic; }
            set { logic = value; }
        }
        private void OnOff()
        {
            while (BluetoothRadio.PrimaryRadio == null)
            {
                ScanText = "BLUETOOTH IS OFF!";

            }
            ScanText = "SCAN";
            logic = new IMViewModelLogic(this);
        }
        public string Connection
        {
            get { return connection; }
            set
            {
                connection = value;
                
                OnPropertyChanged("Connection");
            }
        }
        public string Conversation
        {
            get { return conversation; }
            set { conversation = value;
            OnPropertyChanged("Conversation");
            }
        }

        public string ScanText
        {
            get { return scanText; }
            set { scanText = value;
            OnPropertyChanged("ScanText");
            }
        }



        public string WindowState
        {
            get { return windowState; }
            set
            {
                windowState = value;
                OnPropertyChanged("WindowState");
            }
        }
        public string Enabled
        {
            get { return enabled; }
            set
            {
                enabled = value;
                OnPropertyChanged("Enabled");
            }
        }

        public int ToConnect
        {
            get { return toConnect; }
            set
            {
                toConnect = value;
                OnPropertyChanged("ToConnect");
            }
        }

        public List<string> InRangeCollection
        {
            get { return inRangeCollection; }
            set
            {
               inRangeCollection = new List<string>();
                foreach (string s in value)
                {
                    if (!inRangeCollection.Contains(s))
                        inRangeCollection.Add(s);
                }
                ScanText = "SCAN";
                OnPropertyChanged("InRangeCollection");
            }
        }


        public List<string> RoomMembers
        {
            get { return roomMembers; }
            set
            {
                roomMembers = new List<string>();
                foreach (string s in value)
                {
                    if(!roomMembers.Contains(s))
                    roomMembers.Add(s);
                }
               
                OnPropertyChanged("RoomMembers");
               if(value.Count > 0){                
                    Enabled = "Visible";
                    WindowWidth = 931;
                    SelectedMember = RoomMembers.Last();  
                }
                else
                {
                    InputText = "";
                    WindowWidth = 210;
                    Enabled = "Hidden";
                }
            
               
            }
        }


        public string OutputText
        {
            get { return outputText; }
            set
            {
                outputText = value;

                OnPropertyChanged("OutputText");

            }
        }


        public int WindowWidth
        {
            get { return windowWidth; }
            set
            {
                if(Enabled!="Hidden")
                windowWidth = value;
                OnPropertyChanged("WindowWidth");

            }
        }
        public string SelectedMember
        {
            get { return selectedMember; }
            set
            {
                selectedMember = value;
                OnPropertyChanged("SelectedMember");
                Conversation = value;
                try
                {
                    if (value == "Everyone")
                        InputText = Room.Conversations["Everyone"].InStringConversation;
                    else
                        try
                        {
                            InputText = Room.Conversations[RoutingTable.NameAddressPairs[SelectedMember]].InStringConversation;
                        }
                    catch(Exception)
                        { }
                }
                catch (ArgumentNullException e)
                { }
               
            }
        }


        public string InputText
        {
            get { return inputText; }
            set
            {
                inputText = value;
                OnPropertyChanged("InputText");
            }

        }

    }
}
